import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { BehaviorSubject, Observable, throwError, of } from 'rxjs';
import { map, catchError, tap, delay } from 'rxjs/operators';
import { LoginRequest, LoginResponse, AuthState, UserInfo } from '../interface/auth.interface';

@Injectable({
  providedIn: 'root'
})
export class AuthService {
  private readonly API_URL = 'http://localhost:3000'; // Replace with your actual API domain
  private readonly TOKEN_KEY = 'auth_token';
  private readonly USER_KEY = 'user_info';

  // Reactive state management
  private authStateSubject = new BehaviorSubject<AuthState>({
    isAuthenticated: false,
    user: null,
    token: null
  });

  public authState$ = this.authStateSubject.asObservable();

  constructor(private http: HttpClient) {
    // Load saved auth state on service init
    this.loadAuthState();
  }

  /**
   * Login with username and password
   */
  login(credentials: LoginRequest): Observable<LoginResponse> {
    const headers = new HttpHeaders({
      'Content-Type': 'application/json'
    });

    return this.http.post<LoginResponse>(`${this.API_URL}/login`, credentials, { headers })
      .pipe(
        tap(response => this.handleLoginSuccess(response, credentials.username)),
        catchError(error => this.handleError(error))
      );
  }

  /**
   * Logout user
   */
  logout(): Observable<any> {
    // Clear local storage and auth state immediately
    this.handleLogoutSuccess();
    
    // Return success observable
    return of({ success: true, message: 'Logged out successfully' }).pipe(
      delay(500) // Small delay to show loading state
    );
  }

  /**
   * Refresh access token
   */
  refreshToken(): Observable<LoginResponse> {
    const refreshToken = localStorage.getItem('refresh_token');
    const currentUser = this.authStateSubject.value.user;
    
    if (!refreshToken) {
      this.logout();
      return throwError('No refresh token available');
    }

    return this.http.post<LoginResponse>(`${this.API_URL}/refresh`, { 
      refreshToken 
    }).pipe(
      tap(response => this.handleLoginSuccess(response, currentUser?.username || 'Unknown')),
      catchError(error => {
        this.logout();
        return throwError(error);
      })
    );
  }

  /**
   * Get current user info
   */
  getCurrentUser(): Observable<UserInfo> {
    const headers = this.getAuthHeaders();
    
    return this.http.get<UserInfo>(`${this.API_URL}/me`, { headers })
      .pipe(
        tap(user => this.updateUserInfo(user)),
        catchError(error => this.handleError(error))
      );
  }

  /**
   * Check if user is authenticated
   */
  isAuthenticated(): boolean {
    return this.authStateSubject.value.isAuthenticated;
  }

  /**
   * Check if user has specific role
   */
  hasRole(role: string): boolean {
    const currentUser = this.authStateSubject.value.user;
    return currentUser?.role === role;
  }

  /**
   * Check if user is admin
   */
  isAdmin(): boolean {
    return this.hasRole('admin');
  }

  /**
   * Check if user is customer
   */
  isCustomer(): boolean {
    return this.hasRole('customer');
  }

  /**
   * Get stored token
   */
  getToken(): string | null {
    return localStorage.getItem(this.TOKEN_KEY);
  }

  /**
   * Get current user info
   */
  getUser(): UserInfo | null {
    return this.authStateSubject.value.user;
  }

  /**
   * Get auth headers for API calls
   */
  private getAuthHeaders(): HttpHeaders {
    const token = this.getToken();
    return new HttpHeaders({
      'Authorization': `Bearer ${token}`,
      'Content-Type': 'application/json'
    });
  }

  /**
   * Handle successful login
   */
  private handleLoginSuccess(response: LoginResponse, username: string): void {
    if (!response.success) {
      throw new Error(response.message || 'Login failed');
    }

    const { token, role, expirationTime } = response.responseData;
    
    // Store tokens
    localStorage.setItem(this.TOKEN_KEY, token);
    localStorage.setItem('token_expiration', expirationTime.toString());
    
    // Create user info from response
    const userInfo: UserInfo = {
      username: username, // Use the username from login credentials
      role: role
    };
    
    localStorage.setItem(this.USER_KEY, JSON.stringify(userInfo));

    // Update auth state
    this.authStateSubject.next({
      isAuthenticated: true,
      user: userInfo,
      token: token
    });
  }

  /**
   * Handle logout
   */
  private handleLogoutSuccess(): void {
    // Clear storage
    localStorage.removeItem(this.TOKEN_KEY);
    localStorage.removeItem('refresh_token');
    localStorage.removeItem(this.USER_KEY);

    // Reset auth state
    this.authStateSubject.next({
      isAuthenticated: false,
      user: null,
      token: null
    });
  }

  /**
   * Load auth state from storage
   */
  private loadAuthState(): void {
    const token = localStorage.getItem(this.TOKEN_KEY);
    const userInfo = localStorage.getItem(this.USER_KEY);

    if (token && userInfo) {
      try {
        const user = JSON.parse(userInfo);
        this.authStateSubject.next({
          isAuthenticated: true,
          user,
          token
        });
      } catch (error) {
        this.handleLogoutSuccess();
      }
    }
  }

  /**
   * Update user info
   */
  private updateUserInfo(user: UserInfo): void {
    localStorage.setItem(this.USER_KEY, JSON.stringify(user));
    
    const currentState = this.authStateSubject.value;
    this.authStateSubject.next({
      ...currentState,
      user
    });
  }

  /**
   * Handle API errors
   */
  private handleError(error: any): Observable<never> {
    let errorMessage = 'An error occurred';

    if (error.error?.message) {
      errorMessage = error.error.message;
    } else if (error.message) {
      errorMessage = error.message;
    }

    switch (error.status) {
      case 401:
        errorMessage = 'Invalid credentials';
        this.handleLogoutSuccess();
        break;
      case 403:
        errorMessage = 'Access denied';
        break;
      case 500:
        errorMessage = 'Server error. Please try again later.';
        break;
    }
    return throwError(errorMessage);
  }
}
